#ifndef KYSEC_KID_WHLIST_H__
#define KYSEC_KID_WHLIST_H__

#include <stddef.h>
#include <stdint.h>
#include <sys/queue.h>
#include <linux/limits.h>

#ifndef KID_NAME_LEN
#define KID_NAME_LEN 128
#endif

//KID类型
typedef enum kid_whlist_kid_type_e  {
    kid_type_user = 0,           //用户
    kid_type_core,              //核心
    kid_type_system,              //系统
    kid_type_app,              //应用软件
    kid_type_tmp,              //临时
    kid_type_max    
}kid_whlist_kid_type;

//主体类型
typedef enum kid_whlist_sub_type_e  {
    sub_type_app = 0,           //应用
    sub_type_file,              //文件、程序
    sub_type_max    
}kid_whlist_sub_type;

typedef struct kid_whlist_res_t{
    uint32_t kid_type;                           //kid类型
    uint32_t type;                              //资源类型， 0 普通资源， 1 遍历目录， 2 递归遍历目录
    char resource[PATH_MAX];                 //资源路径    
}kid_whlist_res;

/**
 * @brief Construct a new slist head object
 *        资源列表
 * 
 */
SLIST_HEAD(kid_whlist_res_slist, kid_whlist_kid_res_entry);
struct kid_whlist_kid_res_entry {
	kid_whlist_res res;
	SLIST_ENTRY(kid_whlist_kid_res_entry) entries;
};
typedef struct kid_whlist_res_slist kid_whlist_res_list;

//应用kid数据记录
typedef struct kid_whlist_app_t {
	char source_name[KID_NAME_LEN];			//源码包名
	char app_name[KID_NAME_LEN];			//软件名称
    int type;                               //软件类型
}kid_whlist_app;

/**
 * @brief 单链表结构体定义
 *
 * 应用KID链表
 */
SLIST_HEAD(kid_whlist_app_slist, kid_whlist_app_entry);
struct kid_whlist_app_entry {
	kid_whlist_app app;
	SLIST_ENTRY(kid_whlist_app_entry) entries;
};
typedef struct kid_whlist_app_slist kid_whlist_app_list;

//应用kid策略数据记录
typedef struct kid_whlist_policy_t {
	kid_whlist_sub_type sub_type;
	char subject_name[PATH_MAX];			//软件名称
    char object_name[PATH_MAX];			    //软件名称
    uint32_t perm;
}kid_whlist_policy;

/**
 * @brief 单链表结构体定义
 *
 * 应用KID策略链表
 */
SLIST_HEAD(kid_whlist_policy_slist, kid_whlist_policy_entry);
struct kid_whlist_policy_entry {
	kid_whlist_policy policy;
	SLIST_ENTRY(kid_whlist_policy_entry) entries;
};
typedef struct kid_whlist_policy_slist kid_whlist_policy_list;

//应用kid联网策略数据记录
typedef struct kid_whlist_netctl_t {
	kid_whlist_sub_type sub_type;
	char subject_name[PATH_MAX];			//软件名称    
}kid_whlist_netctl;

/**
 * @brief 单链表结构体定义
 *
 * 应用KID联网策略链表
 */
SLIST_HEAD(kid_whlist_netctl_slist, kid_whlist_netctl_entry);
struct kid_whlist_netctl_entry {
	kid_whlist_netctl netctl;
	SLIST_ENTRY(kid_whlist_netctl_entry) entries;
};
typedef struct kid_whlist_netctl_slist kid_whlist_netctl_list;

#ifdef __cplusplus
extern "C" {
#endif

/**
 * @brief 获取系统权限分级模式
 * 
 * @return int 0 关闭， 1 强制模式（开启权限分级，策略为向量表策略）， 
 *             2  兼容模式（开启权限分级，策略为内核策略和向量表策略）， 其他 失败
 */
int kysec_kid_whlist_func_status_get();

/**
 * @brief 添加系统类别资源
 * 
 * @param status  0 关闭， 1 强制模式（开启权限分级，策略为向量表策略）
 *                2  兼容模式（开启权限分级，策略为内核策略和向量表策略）
 * @return int 0 成功， 其他 失败
 */
int kysec_kid_whlist_func_status_set(int status);

/**
 * @brief 添加资源
 * 
 * @param kid_type 资源类型
 * @param type 资源子类型， 预留字段， 后续拓展资源子类
 * @param path 资源路径
 * @return int 0 成功， 其他 失败
 */
int kysec_kid_whlist_add_res(kid_whlist_kid_type kid_type, int type, const char * path);

/**
 * @brief 删除资源
 * 
 * @param kid_type 资源类型
 * @param type 资源子类型， 预留字段， 后续拓展资源子类
 * @param path 资源路径
 * @return int 0 成功， 其他 失败
 */
int kysec_kid_whlist_remove_res(kid_whlist_kid_type kid_type, int type, const char * path);

/**
 * @brief 加载资源
 * 
 * @param kid_type 资源类型， sub_type_max表示加载所有资源
 * @param type 资源子类型， 预留字段， 后续拓展资源子类
 * @param list 资源列表， 传出
 * @return int 0 成功， 其他 失败
 */
int kysec_kid_whlist_load_res_list(kid_whlist_kid_type kid_type, int type, kid_whlist_res_list *list);

/**
 * @brief 加载资源
 * 
 * @param kid_type 资源类型， sub_type_max表示加载所有资源
 * @param type 资源子类型， 预留字段， 后续拓展资源子类
 * @param list 资源列表， 传出
 * @param size 资源列表长度， 传出
 * @return int 0 成功， 其他 失败
 */
int kysec_kid_whlist_load_res_list_v2(kid_whlist_kid_type kid_type, int type, kid_whlist_res **list, int *size);

/**
 * @brief 获取app或文件kid类型
 * 
 * @param type 传入主体类型
 * @param subject 主体
 * @param kid_type kid类型 传出
 * @return int 0 成功， 其他 失败
 */
int kysec_kid_whlist_get_app_or_file_kid_type(kid_whlist_sub_type type, const char *subject, kid_whlist_kid_type *kid_type);

/**
 * @brief 释放kid资源列表内存
 * 
 * @param list 待释放kid资源信息列表
 */
void kysec_kid_whlist_free_res_list(kid_whlist_res_list *list);

/**
 * @brief 为应用创建kid及数据
 * 
 * @param source_name 源码包名
 * @param app_name    应用名
 * @param files 应用文件列表, 列表为空时，从包列表获取
 * @param size 应用文件列表数量
 * @return int 0 成功， 其他 失败
 */
int kysec_kid_whlist_create_kid(const char *source_name, const char *app_name, const char *files[], int size);

/**
 * @brief 删除应用kid及数据
 * 
 * @param app_name    应用名
 * @param files 应用文件列表
 * @param size 应用文件列表数量
 * @return int 0 成功， 其他 失败
 */
int kysec_kid_whlist_remove_kid(const char *app_name);

/**
 * @brief 提升应用kid等级
 * 
 * @param type  提升类型， 1 核心类型，2 系统类型
 * @param app_name    应用名
 * @param files 应用文件列表
 * @param size 应用文件列表数量
 * @return int 0 成功， 其他 失败
 * @attention 仅支持app类型提升至system类型或core类型
 */
int kysec_kid_whlist_improve_kid(int type, const char *app_name);

/**
 * @brief 加载第三方应用列表
 * 
 * @param list 应用列表， 传出
 * @return int 0 成功， 其他 失败
 */
int kysec_kid_whlist_load_third_app_list(kid_whlist_app_list *list);

/**
 * @brief 加载第三方应用列表
 * 
 * @param list 应用列表， 传出
 * @param size 应用列表长度， 传出
 * @return int 0 成功， 其他 失败
 */
int kysec_kid_whlist_load_third_app_list_v2(kid_whlist_app **list, int *size);

/**
 * @brief 加载系统已安装第三方应用列表
 * 
 * @param list 应用列表， 传出
 * @return int 0 成功， 其他 失败
 */
int kysec_kid_whlist_load_installed_third_app_list(kid_whlist_app_list *list);

/**
 * @brief 加载系统已安装第三方应用列表
 * 
 * @param list 应用列表， 传出
 * @param size 应用列表长度， 传出
 * @return int 0 成功， 其他 失败
 */
int kysec_kid_whlist_load_installed_third_app_list_v2(kid_whlist_app **list, int *size);

/**
 * @brief 加载应用列表
 * 
 * @param list 应用列表， 传出
 * @return int 0 成功， 其他 失败
 */
int kysec_kid_whlist_load_app_list(kid_whlist_app_list *list);

/**
 * @brief 加载应用列表
 * 
 * @param list 应用列表， 传出
 * @param size 应用列表长度， 传出
 * @return int 0 成功， 其他 失败
 */
int kysec_kid_whlist_load_app_list_v2(kid_whlist_app **list, int *size);

/**
 * @brief 加载系统已安装应用列表
 * 
 * @param list 应用列表， 传出
 * @return int 0 成功， 其他 失败
 */
int kysec_kid_whlist_load_installed_app_list(kid_whlist_app_list *list);

/**
 * @brief 加载系统已安装应用列表
 * 
 * @param list 应用列表， 传出
 * @param size 应用列表长度， 传出
 * @return int 0 成功， 其他 失败
 */
int kysec_kid_whlist_load_installed_app_list_v2(kid_whlist_app **list, int *size);

/**
 * @brief 释放kid应用列表内存
 * 
 * @param list 待释放kid应用信息列表
 */
void kysec_kid_whlist_free_app_list(kid_whlist_app_list *list);

/**
 * @brief 添加kid策略
 * 
 * @param sub_type    主体类型
 * @param subject 策略主体
 * @param object 策略客体， 客体对象为目录或文件
 * @param perm 权限， 0x1111， 依次为重命名、删除、写、读
 * @return int 0 成功， 其他 失败
 */
int kysec_kid_whlist_add_kid_policy(kid_whlist_sub_type sub_type, const char *subject, const char *object, int perm);

/**
 * @brief 更新kid策略
 * 
 * @param sub_type    主体类型
 * @param subject 策略主体
 * @param object 策略客体， 客体对象为目录或文件
 * @param perm 权限， 0x1111， 依次为重命名、删除、写、读
 * @return int 0 成功， 其他 失败
 */
int kysec_kid_whlist_update_kid_policy(kid_whlist_sub_type sub_type, const char *subject, const char *object, int perm);

/**
 * @brief 移除kid策略
 * 
 * @param sub_type    主体类型
 * @param subject 策略主体
 * @param object 策略客体， 客体对象为目录或文件
 * @return int 0 成功， 其他 失败
 */
int kysec_kid_whlist_remove_kid_policy(kid_whlist_sub_type sub_type, const char *subject, const char *object);

/**
 * @brief 加载KID策略列表
 * 
 * @param list 策略列表， 传出
 * @return int 0 成功， 其他 失败
 */
int kysec_kid_whlist_load_policy_list(kid_whlist_policy_list *list);

/**
 * @brief 加载KID策略列表
 * 
 * @param list 策略列表， 传出
 * @param size 策略列表长度，
 * @return int 0 成功， 其他 失败
 */
int kysec_kid_whlist_load_policy_list_v2(kid_whlist_policy **list, int *size);

/**
 * @brief 根据主体加载KID策略列表
 * 
 * @param sub_type    主体类型
 * @param subject 策略主体
 * @param list 策略列表， 传出
 * @return int 0 成功， 其他 失败
 */
int kysec_kid_whlist_load_policy_list_by_subject(kid_whlist_sub_type sub_type, const char *subject, kid_whlist_policy_list *list);

/**
 * @brief 根据主体加载KID策略列表
 * 
 * @param sub_type    主体类型
 * @param subject 策略主体
 * @param list 策略列表， 传出
 * @param size 策略列表长度，传出
 * @return int 0 成功， 其他 失败
 */
int kysec_kid_whlist_load_policy_list_by_subject_v2(kid_whlist_sub_type sub_type, const char *subject, kid_whlist_policy **list, int *size);

/**
 * @brief 释放kid应用列表内存
 * 
 * @param list 待释放kid应用信息列表
 */
void kysec_kid_whlist_free_policy_list(kid_whlist_policy_list *list);

/**
 * @brief 添加kid联网策略
 * 
 * @param sub_type    主体类型
 * @param subject 策略主体
 * @return int 0 成功， 其他 失败
 */
int kysec_kid_whlist_add_kid_netctl(kid_whlist_sub_type sub_type, const char *subject);

/**
 * @brief 删除kid联网策略
 * 
 * @param sub_type    主体类型
 * @param subject 策略主体
 * @return int 0 成功， 其他 失败
 */
int kysec_kid_whlist_remove_kid_netctl(kid_whlist_sub_type sub_type, const char *subject);

/**
 * @brief 加载KID联网列表
 * 
 * @param list 应用列表， 传出
 * @return int 0 成功， 其他 失败
 */
int kysec_kid_whlist_load_netctl_list(kid_whlist_netctl_list *list);

/**
 * @brief 加载KID联网列表
 * 
 * @param list 应用列表， 传出
 * @param size 策略列表长度，传出
 * @return int 0 成功， 其他 失败
 */
int kysec_kid_whlist_load_netctl_list_v2(kid_whlist_netctl **list, int *size);

/**
 * @brief 释放kid联网列表内存
 * 
 * @param list 待释放kid应用信息列表
 */
void kysec_kid_whlist_free_netctl_list(kid_whlist_netctl_list *list);

/**
 * @brief 添加用户隐私资源
 * 
 * @param resource 资源路径
 * @param default_perm 默认权限
 * @return int 0 成功， 其他 失败
 */
int kysec_kid_whlist_add_user_privacy_res(const char *resource);

/**
 * @brief 删除用户隐私资源
 * 
 * @param resource 资源路径
 * @return int 0 成功， 其他 失败
 */
int kysec_kid_whlist_remove_user_privcay_res(const char *resource);

#ifdef __cplusplus
}
#endif

#endif


